@@ -2,11 +2,12 @@ import json
 import logging
 import os
 from json import dumps, loads
-from typing import Any, Optional, Union
+from typing import Any, Dict, Optional, Union
 
 from django.apps import apps
 from django.core.exceptions import ImproperlyConfigured
 
+from django_swagger_tester.configuration import settings
 from django_swagger_tester.exceptions import UndocumentedSchemaSectionError
 from django_swagger_tester.openapi import list_types, read_properties
 from django_swagger_tester.utils import Route
@@ -56,9 +57,7 @@ class _LoaderBase:
         """
         Sets self.schema and self.original_schema.
         """
-        self.schema = self.replace_refs(
-            schema,
-        )
+        self.schema = self.replace_refs(schema, loader=self)
         self.original_schema = schema
 
     def get_route(self, route: str) -> Route:
@@ -96,7 +95,17 @@ class _LoaderBase:
         routes = ', '.join(list(paths_schema))
         route_error = ''
         if routes:
-            route_error += f'\n\nFor debugging purposes, other valid routes include: {routes}.'
+            pretty_routes = '\n\t• '.join(routes.split())
+
+            if settings.parameterized_i18n_name:
+                route_error += (
+                    '\n\nDid you specify the correct i18n parameter name? '
+                    f'Your project settings specify `{settings.parameterized_i18n_name}` '
+                    f'as the name of your parameterized language, meaning a path like `/api/en/items` '
+                    f'will be indexed as `/api/{{{settings.parameterized_i18n_name}}}/items`.'
+                )
+            route_error += f'\n\nFor debugging purposes, other valid routes include: \n\n\t• {pretty_routes}'
+
         if 'skip_validation_warning' in kwargs and kwargs['skip_validation_warning']:
             route_error += (
                 f'\n\nTo skip validation for this route you can add `^{route}$` '
@@ -109,7 +118,9 @@ class _LoaderBase:
                 # This is an unfortunate piece of logic, where we're attempting to insert path parameters
                 # one by one until the path works
                 # if it never works, we finally raise an UndocumentedSchemaSectionError
-                route_schema = index_schema(schema=paths_schema, variable=route_object.get_path(), error_addon=route_error)
+                route_schema = index_schema(
+                    schema=paths_schema, variable=route_object.get_path(), error_addon=route_error
+                )
                 break
             except UndocumentedSchemaSectionError as e:
                 error = e
@@ -119,7 +130,8 @@ class _LoaderBase:
             raise error  # type: ignore
 
         # Index by method
-        joined_methods = ', '.join([method.upper() for method in route_schema.keys() if method.upper() != 'PARAMETERS'])
+        joined_methods = ', '.join(method.upper() for method in route_schema.keys() if method.upper() != 'PARAMETERS')
+
         method_error = ''
         if joined_methods:
             method_error += f'\n\nAvailable methods include: {joined_methods}.'
@@ -129,11 +141,13 @@ class _LoaderBase:
         responses_schema = index_schema(schema=method_schema, variable='responses')
 
         # Index by status code
-        responses = ', '.join([f'{code}' for code in responses_schema.keys()])
+        responses = ', '.join(f'{code}' for code in responses_schema.keys())
         status_code_error = f' Is the `{status_code}` response documented?'
         if responses:
             status_code_error = f'\n\nDocumented responses include: {responses}. ' + status_code_error  # reverse add
-        status_code_schema = index_schema(schema=responses_schema, variable=str(status_code), error_addon=status_code_error)
+        status_code_schema = index_schema(
+            schema=responses_schema, variable=str(status_code), error_addon=status_code_error
+        )
 
         # Not sure about this logic - this is what my static schema looks like, but not the drf_yasg dynamic schema
         if 'content' in status_code_schema and 'application/json' in status_code_schema['content']:
@@ -162,11 +176,21 @@ class _LoaderBase:
         routes = ', '.join(list(paths_schema))
         route_error = ''
         if routes:
-            route_error += f'\n\nFor debugging purposes, other valid routes include: {routes}.'
+            pretty_routes = '\n\t• '.join(routes.split())
+
+            if settings.parameterized_i18n_name:
+                route_error += (
+                    '\n\nDid you specify the correct i18n parameter name? '
+                    f'Your project settings specify `{settings.parameterized_i18n_name}` '
+                    f'as the name of your parameterized language, meaning a path like `/api/en/items` '
+                    f'will be indexed as `/api/{{{settings.parameterized_i18n_name}}}/items`.'
+                )
+            route_error += f'\n\nFor debugging purposes, other valid routes include: \n\n\t• {pretty_routes}'
         route_schema = index_schema(schema=paths_schema, variable=route, error_addon=route_error)
 
         # Index by method
-        joined_methods = ', '.join([method.upper() for method in route_schema.keys() if method.upper() != 'PARAMETERS'])
+        joined_methods = ', '.join(method.upper() for method in route_schema.keys() if method.upper() != 'PARAMETERS')
+
         method_error = ''
         if joined_methods:
             method_error += f'\n\nAvailable methods include: {joined_methods}.'
@@ -204,7 +228,9 @@ class _LoaderBase:
         """
         methods = ['get', 'post', 'put', 'patch', 'delete', 'options', 'head']
         if not isinstance(method, str) or method.lower() not in methods:
-            logger.error('Method `%s` is invalid. Should be one of: %s.', method, ', '.join([i.upper() for i in methods]))
+            logger.error(
+                'Method `%s` is invalid. Should be one of: %s.', method, ', '.join([i.upper() for i in methods])
+            )
             raise ImproperlyConfigured(
                 f'Method `{method}` is invalid. Should be one of: {", ".join([i.upper() for i in methods])}.'
             )
@@ -226,7 +252,7 @@ class _LoaderBase:
             raise ImproperlyConfigured('`status_code` should be a valid HTTP response code.')
 
     @staticmethod
-    def replace_refs(schema: dict) -> dict:
+    def replace_refs(schema: dict, loader=None) -> dict:
         """
         Finds all schema references ($ref sections) in an OpenAPI schema and inserts them back into in place of the refs.
         This way we don't have to handle reference section when interacting with a loaded schema.
@@ -234,18 +260,43 @@ class _LoaderBase:
         * This does add a performance overhead to interacting with a schema, so changing this in the future would be fine *
 
         :param schema: OpenAPI schema
+        :param loader: Current loader
         :return Adjusted OpenAPI schema
         """
         if '$ref' not in str(schema):
             return schema
 
+        schemas_cache: Dict[str, dict] = {}
+
         def find_and_replace_refs_recursively(d: dict, s: dict) -> dict:
             """
             Iterates over a dictionary to look for pesky $refs.
             """
             if '$ref' in d:
-                indices = [i for i in d['$ref'][d['$ref'].index('#') + 1 :].split('/') if i]
-                temp_schema = s
+                index_file, _, index_path = d['$ref'].partition('#')
+                # index_path looks like `/GoodTrucksList`
+                # index_file looks like `openapi-schema-split-datastructures.yaml` and will
+                # be blank unless the openapi schema is split into several sections
+                temp_schema: dict
+                if not index_file:
+                    # If index-file is empty, use local
+                    temp_schema = s
+                elif loader:
+                    # If index-file is provided, look in the cache
+                    full_path = os.path.join(os.path.dirname(loader.path), index_file)
+                    schema_from_cache: Optional[dict] = schemas_cache.get(full_path)
+                    if schema_from_cache is not None:
+                        temp_schema = schema_from_cache
+                    else:
+                        # If we cannot find our reference in the cache load external defs
+                        external_loader = loader.__class__()
+                        external_loader.set_path(full_path)
+                        temp_schema = external_loader.get_schema()
+                        schemas_cache[full_path] = temp_schema
+                else:
+                    raise RuntimeError('loader is required for external references')
+
+                indices = [i for i in index_path.split('/') if i]
                 for index in indices:
                     logger.debug('Indexing schema by `%s`', index)
                     temp_schema = temp_schema[index]
@@ -351,6 +402,8 @@ class DrfYasgSchemaLoader(_LoaderBase):
         from drf_yasg.generators import OpenAPISchemaGenerator
         from drf_yasg.openapi import Info
 
+        logger.debug('Initialized drf-yasg loader schema')
+
         self.schema_generator = OpenAPISchemaGenerator(info=Info(title='', default_version=''))
 
     def validation(self, *args, **kwargs) -> None:
@@ -363,7 +416,9 @@ class DrfYasgSchemaLoader(_LoaderBase):
         try:
             import drf_yasg  # noqa: F401
         except ModuleNotFoundError:
-            raise ImproperlyConfigured('The package `drf_yasg` is required. Please run `pip install drf_yasg` to install it.')
+            raise ImproperlyConfigured(
+                'The package `drf_yasg` is required. Please run `pip install drf_yasg` to install it.'
+            )
 
         if 'drf_yasg' not in apps.app_configs.keys():
             raise ImproperlyConfigured(
@@ -419,6 +474,7 @@ class DrfSpectacularSchemaLoader(_LoaderBase):
         from drf_spectacular.generators import SchemaGenerator
 
         self.schema_generator = SchemaGenerator()
+        logger.debug('Initialized drf-spectacular loader schema')
 
     def validation(self, *args, **kwargs) -> None:
         """
@@ -477,6 +533,7 @@ class StaticSchemaLoader(_LoaderBase):
     def __init__(self) -> None:
         super().__init__()
         self.path: str = ''
+        logger.debug('Initialized static loader schema')
 
     def set_path(self, path: str) -> None:
         """
